/******************************************************************************
*
* Pacman 3D: For the 89, 92+, and v200
* By Malcolm Smith
* m@malcolm-s.net
* Started on August 20th, 2002
*
******************************************************************************/

//Quitting:
//-1: Died
//1: Pressed Esc
//2: Beat all internal levels or the user level.


#define USE_TI89
#define USE_TI92P
#define USE_V200

#include "includes.h" //Include everything

void _main(void) {

    TEXCONFIG       textures[2];
    LCD_BUFFER      lcd;
    HANDLE          texfile = 0;
    unsigned char*  src     = 0;
    short           result;

    FATCONFIG fc = {
        20,         // width of map
        22,				  // height of map
        map[0],     // pointer to map data
        textures,   // pointer to textures array
        1,          // number of sprites
        NULL,       // the sprite array
        96,         // start pos_x
        96,         // start pos_y
        ANGLE_EAST, // orientation
        NULL,       // destination plane0
        NULL,       // destination plane1
        BG_name,    // background image
        0           // frame counter
    };

    //-------------------------------------------------------------------------
    // before we try to do anything else we have to initialize the engine
    // its only valid to proceed if FAT_LOADOKAY gets returned
    //-------------------------------------------------------------------------
    if ((result = FAT_LoadEngine()) != FAT_LOADOKAY) {
        switch(result) {
            case FAT_NOTGHOSTSPACE: ST_helpMsg("ERROR: client not exepacked!");break;
            case FAT_NOTFOUND:      ST_helpMsg("ERROR: no matching engine found");break;
            case FAT_LOCKFAILED:    ST_helpMsg("ERROR: locking engine failed");break;
            case FAT_OUTOFMEM:      ST_helpMsg("ERROR: out of memory");break;
            default:                ST_helpMsg("ERROR: unknown errorcode");break;
        }
        return;
    }

	  //-------------------------------------------------------------------------
    // try to load the used textures from GRAPHICS_FILE_NAME ...
    //-------------------------------------------------------------------------
    if ((texfile = FAT_GetHandleOfFile(GRAPHICS_FILE_NAME))) {
        if ((src = FAT_LockHandleOfFile(texfile))) {
            // load textures
            if (!FAT_LoadTextures(textures,src+2,TEXIDX_STONE1,1)) {
                src = 0; // just to indicate something went wrong
            }
            if (!FAT_LoadTextures(sprite_tex,src+2,TEXIDX_BALL1,4)) { //Load sprites
                src = 0; // just to indicate something went wrong
            }
           
           if (!(pac_img_data = FAT_GetGenericData(src+2,GDIDX_PACLOGO,4000))) { //Load logo
                src = 0; // just to indicate something went wrong
            }
        }
    }

    //-------------------------------------------------------------------------
    // check if loading failed ...
    //-------------------------------------------------------------------------
    if (!texfile || !src) {
        FAT_UnloadEngine();
        if (texfile) FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: Can't find textures from pac_gfx");
        return;
    }

    LCD_save(lcd);
    
    InitGameStruct();
		
		LoadConfig();
		
		allspr=(FATSPRITE*)malloc(GHOST_LIMIT*sizeof(FATSPRITE) +  MAX_ONSCREEN_DOTS*sizeof(FATSPRITE));
		if(allspr==NULL)
		{
        RestoreHandlerForINTs();
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: out of mem");
        return;			
		}
		
		fc.sprites = allspr;

    InstallDummyHandlerForINTs();
    
    gamedata.quitting = 0;

    //-------------------------------------------------------------------------
    // initialize grayscales, clear both planes and set the destination plane
    // addresses for the FAT-Engine
    //-------------------------------------------------------------------------
    if (!GrayOn()) {
        RestoreHandlerForINTs();
        FAT_UnloadEngine();
        FAT_UnlockHandleOfFile(texfile);
        ST_helpMsg("ERROR: out of mem");
        return;
    }

    //-------------------------------------------------------------------------
    // display the FAT-Engine's splashscreen
    //-------------------------------------------------------------------------
    if(gamedata.cfg.fatlogo) FAT_DisplayLogo(30);

    fc.dest_plane0 = GetPlane(0)+30*2;  // center vertically
    fc.dest_plane1 = GetPlane(1)+30*2;  // center vertically
	
		//Display the Pacman 3D Logo, handle the menu, etc
		gamedata.quitting = DrawPacmanLogoAndHandleMenu(&fc);

    //-------------------------------------------------------------------------
    // setup calculator type specials
    //-------------------------------------------------------------------------
    if (TI89) {
        display_offsetx = 0;
        display_offsety = 0;
    }
    else {
        // TI92p or V200
        display_offsetx  = 40;
        display_offsety  = 14;
        fc.dest_plane0  += (14*30+4)/2;
        fc.dest_plane1  += (14*30+4)/2;
    }

    //-------------------------------------------------------------------------
    // initialize the GUI
    //-------------------------------------------------------------------------

    SetGameVariables(&fc);
    if(!gamedata.quitting)
    {
    InitGUI();
    DrawMinimap(&fc);	
    }
		int5_counter = 0;
		count5_start = int5_counter;

    //-------------------------------------------------------------------------
    // the main rendering loop ...
    //-------------------------------------------------------------------------
    while(!gamedata.quitting) {
    	 CheckGhostCollision(&fc);
       CheckGhosts(&fc);
       fc.nr_sprites = gamedata.ghosts.count;
       fc.nr_sprites += CheckDotPresent(&fc);
       CheckDotEaten(&fc);
       FAT_Render(&fc);
/*
{
	char buf[30];
	sprintf(buf,"%i %i %i      ",fc.nr_sprites,gamedata.ghosts.count,fc.nr_sprites-gamedata.ghosts.count);;
DrawGrayStrExt(97+display_offsetx,16+display_offsety,buf,F_4x6,A_REVERSE);	
}*/
       CheckPlayerMinimapUpdate(&fc);
       //UpdateGhostsOnMinimap(&fc);
    	 if(!HandlePlayerMovement(&fc)) break;   

 
    }
if(gamedata.quitting==WON_QUIT_LEVEL) ShowCongratulationsScreen();

    //-------------------------------------------------------------------------
    // some clean stuff ...
    //-------------------------------------------------------------------------
    FAT_AllKeysReleased();
    GrayOff();
    SaveConfig();
    LCD_restore(lcd);
    if(allspr!=NULL) free(allspr);
    if(ext_level_data!=NULL) free(ext_level_data);
    ST_helpMsg(FAT_GetPoweredByString());
    FAT_UnlockHandleOfFile(texfile);  // unlock texture file
    FAT_UnloadEngine();
    RestoreHandlerForINTs();
    OSqclear(kbd_queue());
}

